#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>
#include <dirent.h>
#include <string.h>
#include <time.h>

#define PATH_SIZE 256

/* Na pocetku su imena fajlova prazne niske a njihovo vreme zadnje izmene Epoha */
char file1[PATH_SIZE] = "";
char file2[PATH_SIZE] = "";
char file3[PATH_SIZE] = "";
time_t max1;
time_t max2;
time_t max3;


void traverse (char *file_name);
void error_fatal (const char *message);
void error_fatal_file (const char *message, const char *file_name);

int main(int argc, char **argv){
  assert (argc == 2);

  traverse (argv[1]);
  printf ("%s %s\n", file1, ctime (&max1));
  printf ("%s %s\n", file2, ctime (&max2));
  printf ("%s %s\n", file3, ctime (&max3));
  return 0;
}

void traverse (char *file_name){
  struct stat file_info;

  /* Sakupljaju se informacije o fajlu pomocu lstat funkcije. */
  if (lstat (file_name, &file_info) < 0)
    error_fatal_file ("lstat error: ", file_name);

  /* Proverava se da li fajl koji se trenutno obradjuje jeste medju dosadasnjih
     3 najskorije menjanih. Ako jeste vrse se odgovarajuce zamene stringova i 
     maksimuma. Obratiti paznju da ako je fajl menjan, onda je i direktorijum u
     kome se nalazi fajl menjan. */
  if (file_info.st_mtime > max1){
    max3 = max2;
    strcpy (file3, file2);
    max2 = max1;
    strcpy (file2, file1);
    max1 = file_info.st_mtime;
    strcpy (file1, file_name);
  }
  else if (file_info.st_mtime > max2){
    max3 = max2;
    strcpy (file3, file2);
    max2 = file_info.st_mtime;
    strcpy (file2, file_name);
  }
  else if (file_info.st_mtime > max3){
    max3 = file_info.st_mtime;
    strcpy (file3, file_name);
  }

  if (S_ISDIR (file_info.st_mode)){
    DIR *directory;
    struct dirent *entry;

    /* Ako se naislo na direktorijum, mora se proci i kroz sve fajlove koji se u njemu nalaze. */

    /* Otvaranje direktorijuma. */
    if ( (directory = opendir (file_name)) < 0) 
      error_fatal_file ("opendir error", file_name);

    /* Citanje direktorijumskih odrednica. */
    while ( (entry = readdir (directory)) != NULL){

      /* Ako se naidje na odrednicu . ili .. preskace se (u suprotnom bi postojala beskonacna petlja.  */
      if (strcmp (entry->d_name, ".") == 0 || strcmp (entry->d_name, "..") == 0)
	continue;

      /* Alocira se neophodan memorijski prostor za ime fajla u direktorijumu. */
      char *buffer = malloc ( (strlen (file_name) + 1 + strlen (entry->d_name) + 1) * sizeof (char));
      strcpy (buffer, file_name);
      strcat (buffer, "/");
      strcat (buffer, entry->d_name);

      /* Rekurzivni poziv funkije za fajl u direktorijumu. */
      traverse (buffer);

      /* VAZNO: dealokacija memorije. */
      free (buffer);
    }

    if (closedir (directory) < 0)
      error_fatal_file ("closedir error", file_name);
  }
}

/* Funkcija za obradu greske. */
void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

/* Funkcija za obradu greske. */
void error_fatal_file (const char *message, const char *file_name){
  perror (message);
  fprintf (stderr, "%s", file_name);
  exit (EXIT_FAILURE);
}

